@@ -20,6 +20,9 @@
 /* we add that little bit to avoid rounding issues */
 #define SampleFTo16(num) ((int)((num) * 65535.0 + 0.01))
 
+/* maximum size of an SGI image */
+#define SGI_DIM_LIMIT 0xFFFF
+
 typedef struct {
   unsigned short imagic;
   unsigned char storagetype;
@@ -121,7 +124,7 @@ store_16(unsigned char *buf, unsigned short value) {
 }
 
 static void
-store_32(unsigned char *buf, unsigned short value) {
+store_32(unsigned char *buf, unsigned long value) {
   buf[0] = value >> 24;
   buf[1] = (value >> 16) & 0xFF;
   buf[2] = (value >> 8) & 0xFF;
@@ -325,6 +328,11 @@ i_writesgi_wiol(io_glue *ig, i_img *img) {
 
   i_clear_error();
 
+  if (img->xsize > SGI_DIM_LIMIT || img->ysize > SGI_DIM_LIMIT) {
+    i_push_error(0, "image too large for SGI");
+    return 0;
+  }
+
   if (!write_sgi_header(img, ig, &rle, &bpc2))
     return 0;
 
@@ -987,9 +995,8 @@ write_sgi_8_rle(i_img *img, io_glue *ig) {
 	  
 	  /* fill out the run if 2 or less samples left and there's space */
 	  if (in_left - run_length <= 2 
-	      && run_length + in_left - run_length <= 127) {
-	    run_length += in_left;
-	    in_left = 0;
+	      && in_left <= 127) {
+	    run_length = in_left;
 	  }
 	  in_left -= run_length;
 	  *outp++ = run_length | 0x80;
@@ -1145,9 +1152,8 @@ write_sgi_16_rle(i_img *img, io_glue *ig) {
 	  
 	  /* fill out the run if 2 or less samples left and there's space */
 	  if (in_left - run_length <= 2 
-	      && run_length + in_left - run_length <= 127) {
-	    run_length += in_left;
-	    in_left = 0;
+	      && in_left <= 127) {
+	    run_length = in_left;
 	  }
 	  in_left -= run_length;
 	  store_16(outp, run_length | 0x80);
