@@ -4,13 +4,18 @@
 /* keep this file simple - apidocs.perl parses it. */
 
 #include "imdatatypes.h"
+#include <stdarg.h>
 
 /*
  IMAGER_API_VERSION is similar to the version number in the third and
  fourth bytes of TIFF files - if it ever changes then the API has changed
  too much for any application to remain compatible.
+
+ Version 2 changed the types of some parameters and pointers.  A
+ simple recompile should be enough in most cases.
+
 */
-#define IMAGER_API_VERSION 1
+#define IMAGER_API_VERSION 2
 
 /*
  IMAGER_API_LEVEL is the level of the structure.  New function pointers
@@ -25,36 +30,36 @@ typedef struct {
   int level;
 
   /* IMAGER_API_LEVEL 1 functions */
-  void * (*f_mymalloc)(int size);
+  void * (*f_mymalloc)(size_t size);
   void (*f_myfree)(void *block);
   void * (*f_myrealloc)(void *block, size_t newsize);
   void* (*f_mymalloc_file_line)(size_t size, char* file, int line);
   void  (*f_myfree_file_line)(void *p, char*file, int line);
   void* (*f_myrealloc_file_line)(void *p, size_t newsize, char* file,int line);
 
-  i_img *(*f_i_img_8_new)(int xsize, int ysize, int channels);
-  i_img *(*f_i_img_16_new)(int xsize, int ysize, int channels);
-  i_img *(*f_i_img_double_new)(int xsize, int ysize, int channels);
-  i_img *(*f_i_img_pal_new)(int xsize, int ysize, int channels, int maxpal);
+  i_img *(*f_i_img_8_new)(i_img_dim xsize, i_img_dim ysize, int channels);
+  i_img *(*f_i_img_16_new)(i_img_dim xsize, i_img_dim ysize, int channels);
+  i_img *(*f_i_img_double_new)(i_img_dim xsize, i_img_dim ysize, int channels);
+  i_img *(*f_i_img_pal_new)(i_img_dim xsize, i_img_dim ysize, int channels, int maxpal);
   void (*f_i_img_destroy)(i_img *im);
-  i_img *(*f_i_sametype)(i_img *im, int xsize, int ysize);
-  i_img *(*f_i_sametype_chans)(i_img *im, int xsize, int ysize, int channels);
-  void (*f_i_img_info)(i_img *im, int *info);
+  i_img *(*f_i_sametype)(i_img *im, i_img_dim xsize, i_img_dim ysize);
+  i_img *(*f_i_sametype_chans)(i_img *im, i_img_dim xsize, i_img_dim ysize, int channels);
+  void (*f_i_img_info)(i_img *im, i_img_dim *info);
 
-  int (*f_i_ppix)(i_img *im, int x, int y, const i_color *val);
-  int (*f_i_gpix)(i_img *im, int x, int y, i_color *val);
-  int (*f_i_ppixf)(i_img *im, int x, int y, const i_fcolor *val);
-  int (*f_i_gpixf)(i_img *im, int x, int y, i_fcolor *val);
-  int (*f_i_plin)(i_img *im, int l, int r, int y, const i_color *vals);
-  int (*f_i_glin)(i_img *im, int l, int r, int y, i_color *vals);
-  int (*f_i_plinf)(i_img *im, int l, int r, int y, const i_fcolor *vals);
-  int (*f_i_glinf)(i_img *im, int l, int r, int y, i_fcolor *vals);
-  int (*f_i_gsamp)(i_img *im, int l, int r, int y, i_sample_t *samp, 
+  int (*f_i_ppix)(i_img *im, i_img_dim x, i_img_dim y, const i_color *val);
+  int (*f_i_gpix)(i_img *im, i_img_dim x, i_img_dim y, i_color *val);
+  int (*f_i_ppixf)(i_img *im, i_img_dim x, i_img_dim y, const i_fcolor *val);
+  int (*f_i_gpixf)(i_img *im, i_img_dim x, i_img_dim y, i_fcolor *val);
+  i_img_dim (*f_i_plin)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_color *vals);
+  i_img_dim (*f_i_glin)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_color *vals);
+  i_img_dim (*f_i_plinf)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, const i_fcolor *vals);
+  i_img_dim (*f_i_glinf)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_fcolor *vals);
+  i_img_dim (*f_i_gsamp)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_sample_t *samp, 
                    const int *chans, int chan_count);
-  int (*f_i_gsampf)(i_img *im, int l, int r, int y, i_fsample_t *samp, 
+  i_img_dim (*f_i_gsampf)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_fsample_t *samp, 
                    const int *chans, int chan_count);
-  int (*f_i_gpal)(i_img *im, int x, int r, int y, i_palidx *vals);
-  int (*f_i_ppal)(i_img *im, int x, int r, int y, const i_palidx *vals);
+  i_img_dim (*f_i_gpal)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y, i_palidx *vals);
+  i_img_dim (*f_i_ppal)(i_img *im, i_img_dim x, i_img_dim r, i_img_dim y, const i_palidx *vals);
   int (*f_i_addcolors)(i_img *im, const i_color *colors, int count);
   int (*f_i_getcolors)(i_img *im, int i, i_color *, int count);
   int (*f_i_colorcount)(i_img *im);
@@ -68,12 +73,12 @@ typedef struct {
 
   i_fill_t *(*f_i_new_fill_hatch)(const i_color *fg, const i_color *bg, int combine, 
                                   int hatch, const unsigned char *cust_hatch, 
-                                  int dx, int dy);
+                                  i_img_dim dx, i_img_dim dy);
   i_fill_t *(*f_i_new_fill_hatchf)(const i_fcolor *fg, const i_fcolor *bg, int combine, 
                                   int hatch, const unsigned char *cust_hatch, 
-                                  int dx, int dy);
-  i_fill_t *(*f_i_new_fill_image)(i_img *im, const double *matrix, int xoff, 
-                                int yoff, int combine);
+                                  i_img_dim dx, i_img_dim dy);
+  i_fill_t *(*f_i_new_fill_image)(i_img *im, const double *matrix, i_img_dim xoff, 
+                                i_img_dim yoff, int combine);
   i_fill_t *(*f_i_new_fill_fount)(double xa, double ya, double xb, double yb, 
                  i_fountain_type type, i_fountain_repeat repeat, 
                  int combine, int super_sample, double ssample_param, 
@@ -117,30 +122,30 @@ typedef struct {
   int (*f_i_tags_set_color)(i_img_tags *tags, char const *name, int code,
                             i_color const *value);
 
-  void (*f_i_box)(i_img *im, int x1, int y1, int x2, int y2, const i_color *val);
-  void (*f_i_box_filled)(i_img *im, int x1, int y1, int x2, int y2, const i_color *val);
-  void (*f_i_box_cfill)(i_img *im, int x1, int y1, int x2, int y2, i_fill_t *fill);
-  void (*f_i_line)(i_img *im, int x1, int y1, int x2, int y2, const i_color *val, int endp);
-  void (*f_i_line_aa)(i_img *im, int x1, int y1, int x2, int y2, const i_color *val, int endp);
-  void (*f_i_arc)(i_img *im, int x, int y, float rad, float d1, float d2, const i_color *val);
+  void (*f_i_box)(i_img *im, i_img_dim x1, i_img_dim y1, i_img_dim x2, i_img_dim y2, const i_color *val);
+  void (*f_i_box_filled)(i_img *im, i_img_dim x1, i_img_dim y1, i_img_dim x2, i_img_dim y2, const i_color *val);
+  void (*f_i_box_cfill)(i_img *im, i_img_dim x1, i_img_dim y1, i_img_dim x2, i_img_dim y2, i_fill_t *fill);
+  void (*f_i_line)(i_img *im, i_img_dim x1, i_img_dim y1, i_img_dim x2, i_img_dim y2, const i_color *val, int endp);
+  void (*f_i_line_aa)(i_img *im, i_img_dim x1, i_img_dim y1, i_img_dim x2, i_img_dim y2, const i_color *val, int endp);
+  void (*f_i_arc)(i_img *im, i_img_dim x, i_img_dim y, double rad, double d1, double d2, const i_color *val);
   void (*f_i_arc_aa)(i_img *im, double x, double y, double rad, double d1, double d2, const i_color *val);
-  void (*f_i_arc_cfill)(i_img *im, int x, int y, float rad, float d1, float d2, i_fill_t *val);
+  void (*f_i_arc_cfill)(i_img *im, i_img_dim x, i_img_dim y, double rad, double d1, double d2, i_fill_t *val);
   void (*f_i_arc_aa_cfill)(i_img *im, double x, double y, double rad, double d1, double d2, i_fill_t *fill);
-  void (*f_i_circle_aa)(i_img *im, float x, float y, float rad, const i_color *val);
-  int (*f_i_flood_fill)(i_img *im, int seedx, int seedy, const i_color *dcol);
-  int (*f_i_flood_cfill)(i_img *im, int seedx, int seedy, i_fill_t *fill);
+  void (*f_i_circle_aa)(i_img *im, double x, double y, double rad, const i_color *val);
+  int (*f_i_flood_fill)(i_img *im, i_img_dim seedx, i_img_dim seedy, const i_color *dcol);
+  int (*f_i_flood_cfill)(i_img *im, i_img_dim seedx, i_img_dim seedy, i_fill_t *fill);
 
-  void (*f_i_copyto)(i_img *im, i_img *src, int x1, int y1, int x2, int y2, int tx, int ty);
-  void (*f_i_copyto_trans)(i_img *im, i_img *src, int x1, int y1, int x2, int y2, int tx, int ty, const i_color *trans);
+  void (*f_i_copyto)(i_img *im, i_img *src, i_img_dim x1, i_img_dim y1, i_img_dim x2, i_img_dim y2, i_img_dim tx, i_img_dim ty);
+  void (*f_i_copyto_trans)(i_img *im, i_img *src, i_img_dim x1, i_img_dim y1, i_img_dim x2, i_img_dim y2, i_img_dim tx, i_img_dim ty, const i_color *trans);
   i_img *(*f_i_copy)(i_img *im);
-  int (*f_i_rubthru)(i_img *im, i_img *src, int tx, int ty, int src_minx, int src_miny, int src_maxx, int src_maxy);
+  int (*f_i_rubthru)(i_img *im, i_img *src, i_img_dim tx, i_img_dim ty, i_img_dim src_minx, i_img_dim src_miny, i_img_dim src_maxx, i_img_dim src_maxy);
 
   /* IMAGER_API_LEVEL 2 functions */
-  int (*f_i_set_image_file_limits)(int width, int height, int bytes);
-  int (*f_i_get_image_file_limits)(int *width, int *height, int *bytes);
-  int (*f_i_int_check_image_file_limits)(int width, int height, int channels, int sample_size);
-  int (*f_i_flood_fill_border)(i_img *im, int seedx, int seedy, const i_color *dcol, const i_color *border);
-  int (*f_i_flood_cfill_border)(i_img *im, int seedx, int seedy, i_fill_t *fill, const i_color *border);
+  int (*f_i_set_image_file_limits)(i_img_dim width, i_img_dim height, size_t bytes);
+  int (*f_i_get_image_file_limits)(i_img_dim *width, i_img_dim *height, size_t *bytes);
+  int (*f_i_int_check_image_file_limits)(i_img_dim width, i_img_dim height, int channels, size_t sample_size);
+  int (*f_i_flood_fill_border)(i_img *im, i_img_dim seedx, i_img_dim seedy, const i_color *dcol, const i_color *border);
+  int (*f_i_flood_cfill_border)(i_img *im, i_img_dim seedx, i_img_dim seedy, i_fill_t *fill, const i_color *border);
 
   /* IMAGER_API_LEVEL 3 functions */
   void (*f_i_img_setmask)(i_img *im, int ch_mask);
@@ -158,9 +163,9 @@ typedef struct {
   /* IMAGER_API_LEVEL 5 functions will be added here */
   /* added i_psampf?_bits macros */
   int (*f_i_img_is_monochrome)(i_img *, int *zero_is_white);
-  int (*f_i_gsamp_bg)(i_img *im, int l, int r, int y, i_sample_t *samples,
+  int (*f_i_gsamp_bg)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_sample_t *samples,
 		      int out_channels, i_color const * bg);
-  int (*f_i_gsampf_bg)(i_img *im, int l, int r, int y, i_fsample_t *samples,
+  int (*f_i_gsampf_bg)(i_img *im, i_img_dim l, i_img_dim r, i_img_dim y, i_fsample_t *samples,
 		      int out_channels, i_fcolor const * bg);
   void (*f_i_get_file_background)(i_img *im, i_color *bg);
   void (*f_i_get_file_backgroundf)(i_img *im, i_fcolor *bg);
